function ising(action)
%
% Programm fuer das 2D Ising Modell   
% 
% Author:  Winfried Kernbichler,Dieter Mayer
% Version: 1.0 
% Date:    5. November 2001
% Status:  erste Schritte
% Missing:  
         

% Test of all gui_routines

% ==========================================================================
% ==========================================================================
% one should keep this at the beginning of the file
% just change func_name
func_name = 'ising';                    % function name - same as declaration
% ==========================================================================
handle_name = [func_name,'_HANDLE'];  % name of global variable, erzeugt den String: gol_Handle
eval(['global ',handle_name]);        % set it global
if nargin == 0, action = 'initgraph'; end  % set to initgraph without input, wenn keine Eingabeparameter existieren
if ~strcmp(action,'initgraph'),       % vergleicht die Action mit dem Wort initgraph
  eval(['HANDLE=',handle_name,';']);  % copy global to local
else                                  % wenn die Aktion initgraph ist
  % one can change this strings, Grundeinstellungen
  logo_switch = 1;                  % logo: yes (1), no (0)
  fig_title   = '2D Ising Modell';
  logo_title  = '2D Ising Modell';
  logo_name   = 'Winfried Kernbichler, Dieter Mayer';
  logo_lv     = 'Multimediale Lehre';
  logo_type   = 'logo_1';           % only this is defined at the moment
  min_dim     = 5;                  % minimum dimension (x- and y-dir)
  max_dim     = 250;                % maximum dimension (x- and y-dir)
  sta_dim     = 50;                 % standard dimension (x- and y-dir)
  min_iter    = 1;                  % minimum number of iterations
  max_iter    = 1000;               % maximum number of iterations
  sta_iter    = 100;                % standard number of iterations
  min_temp    = 0;                  % minimale Temperatur
  max_temp    = 5;                  % maximale Temperatur
  sta_temp    = 1.5;                % Standardeinstellung fuer die Temperatur
  % some colors
  ax_color = [0.9,0.9,0.9];         % default axes color
  bo_color = [0.6,0.6,0.6];         % default border color
  % do not change
  HANDLE = [];                      % empty at the beginning
end
% ==========================================================================
% ==========================================================================
% Here every aspect of the graphics part should be handled
% This contains the definition and the callback routines
switch lower(action)
  % ========================================================================
  % initialization of program
  case 'initgraph'    
    % ----------------------------------------------------------------------
    % preprocessing for initgraph (keep this lines) 
    eval( [func_name,' ''initgraph_pre'''] );
    % ----------------------------------------------------------------------
    
    % ---------------------------------------------------------------------- 
    % definition and creation of graphics objects
    % ----------------------------------------------------------------------

    % ----------------------------------------------------------------------  
    % definition of main figure
    % ----------------------------------------------------------------------  
    F.figure = struct( ...
	'frame',    { [0.0, 0.0, 1.0, 1.0] }, ...   %Haupt-Frame soll ganzen Bildschrim ausfuellen
	'geometry', { [1] }, ...                    %nur ein Fenster
	'closerequestfcn', { 'close_request_main' }, ...        
	'renderer', { 'opengl' }, ...
	'name',     { fig_title }, ...
	'tag',      { 'main' } ...                  %Kennzeichnung, um figure wiederzuerkennen
	);
    % --- should be kept ---------------------------------------------------  
    [HANDLE,fig_han] = gui_create(gui_define(F),HANDLE); % creation
    clear F;                                      % not needed any more
    % add function name to tag and closerequestfcn
    % important for finding program related figures and to find function
    gui_add_name(fig_han,func_name,'_','b','tag');
    gui_add_name(fig_han,func_name,' ','b','closerequestfcn');
    % sets necessary userdata for figures
    gui_set_userdata(fig_han, ...
	'function',func_name, ...
	'end','exit');
    figmain_han = fig_han;
    % figure color
    fig_color = get( ...
	gui_handle_query(HANDLE.figure,'tag','*main*'),'color');
    % ----------------------------------------------------------------------
    
    % ----------------------------------------------------------------------  
    % create meta frames, Erschaffung der 5 Bildschirm
    % ----------------------------------------------------------------------  
    if logo_switch
      logo_y_pos  = 0.9; % y division between logo and rest
    else
      logo_y_pos  = 1.0;
    end
    meta_frame_logo   = [0, logo_y_pos, 1, 1-logo_y_pos]; % logo
    meta_frame_layer1 = [0, 0,          1, logo_y_pos  ]; % layer 1   
    % ----------------------------------------------------------------------  

    % ----------------------------------------------------------------------  
    % create logo
    % ----------------------------------------------------------------------
    if logo_switch
      % --------------------------------------------------------------------  
      logo_string = {logo_title,logo_name,logo_lv};
      logo_parent = figmain_han;
      % here one could use HANDLE instead of HANDLE_LOGO, then one could 
      % refere to all contents within the rest of the program.
      % When using HANDLE_LOGO this graphics part stays seperate. 
      HANDLE_LOGO = [];
      HANDLE_LOGO = gui_create_logo(HANDLE_LOGO, logo_string, ...
	  logo_parent, meta_frame_logo, logo_type);
    end
    % ----------------------------------------------------------------------  
    % ----------------------------------------------------------------------  

    % ---------------------------------------------------------------------- 
    % create main frames at 4 positions
    %  four joined parts with division lines given at 
    %  x: x_div_pos y: y_div_pos
    % ----------------------------------------------------------------------  
    mf = meta_frame_layer1;    % choose meta frame
    x_div_pos = 0.80 * mf(3);  % division line in x direction
    y_div_pos = 0.20 * mf(4);  % division line in y direction
    % frames for lower-left, lower-right, upper-left, upper-right
    ll_frame = [ mf(1),           mf(2),             ...
	  x_div_pos,       y_div_pos ];
    lr_frame = [ x_div_pos,       mf(2) ,            ...
	  mf(3)-x_div_pos, y_div_pos ];
    ul_frame = [ mf(1),           y_div_pos,         ...
	  x_div_pos,       mf(4)-y_div_pos ];
    ur_frame = [ x_div_pos,       y_div_pos,         ...
	  mf(3)-x_div_pos, mf(4)-y_div_pos ];
	   
    % ----------------------------------------------------------------------  
    % definition of some basis axes system for the background of all four
    % parts defined in the meta frame.
    Fbasis.axes = struct( ...
	'geometry', { [1] }, ...
	'xborder',  { [0.0, 0.0] }, ...
	'yborder',  { [0.0, 0.0] }, ...
	'xgap',     { [0.0] }, ...
	'ygap',     { [0.0] }, ...	
	'parent',   { [1] }, ...
	'visible',  { 'on' }, ...
	'box',      { 'on' }, ...
	'xtick',    { [] }, ...
	'ytick',    { [] }, ...
	'color',    { ax_color }, ...
	'xcolor',   { bo_color }, ...
	'ycolor',   { bo_color } ...	
	);
    
    % ----------------------------------------------------------------------  
    % lower left position (containing sliders)
    % ---------------------------------------------------------------------- 
    % basic axis for background
    Fbasis.axes = setfield(Fbasis.axes,'frame',ll_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);
    gui_add_name(newh,'lower_left',' ','b','tag');            
    
    % definitions of slider values
    geom = [ 1, 1, 1 ; 1, 1, 1 ];
    % typical values
    visible = { 'on', 'on', 'on'; 'on', 'on',  'on'  };
    minval  = { min_dim, min_dim, min_iter;  0,   0,  min_temp  }; % minimum
    maxval  = { max_dim, max_dim, max_iter;  1,   1,  max_temp  }; % maximum
    svalue  = { sta_dim, sta_dim, sta_iter;  0.5, 0,  sta_temp  }; % value for slider
    evalue = svalue;                            % value for edit (string)
    for k = 1:length(svalue(:)), evalue{k} = num2str(svalue{k}); end

    %tag = 'slider'; Sliderbezeichnung, zugleich Variablen Name
    cab = 'set_value';
    str = { ...
	'X_Dimension','Y_Dimension','Iterations'; ...
        'initial_magnet', 'Time_Delay' ,'Temperature'  ...
	};
    tag = lower(str);
    % create variables from string
    var = transpose(flipud(lower(str))); % variables
    % here transpose and flipud is necessary to allow later use with
    % gui_add_userdata 
    % graphics obejects are counted from lower left corner along the 
    % second dimension, arrays and cells from upper left corner along
    % the first dimension.
    % The command lower is used to make sure variable names are lower
    % case.

    % callback action to be performed, fragt, ob bei Betaetigung die Graphik erneuert werden muss
    cb_ac = 'setup_imagemat'; cb_no = '';
    cbaction = { cb_ac, cb_ac, cb_no; cb_ac, cb_no, cb_no };

    cbaction = transpose(flipud(lower(cbaction)));
    % what should be changed in the slider, rundet Sliderwerte auf ganze (bzw gerade) Zahlen
    cb_ch1 = { 'value','round(##)' }; cb_ch2 = { 'value','2*round(##/2)' };cb_no = {};
    cbchange_sl = { cb_ch2, cb_ch2, cb_ch1; cb_no, cb_no, cb_no }; 
    cbchange_sl = transpose(flipud(cbchange_sl));
    % what should be changed in the edit field, rundet Werte in den Edit-Feldern auf ganze bzw. gerade Zahlen
    cb_ch1 = { 'string','num2str((round(str2num(##))))' };cb_ch2 = { 'string','num2str(2*(round(str2num(##)/2)))' };cb_no = {};
    cbchange_ed = { cb_ch2, cb_ch2, cb_ch1; cb_no, cb_no, cb_no };

    cbchange_ed = transpose(flipud(cbchange_ed));
    % what should be updated when slider changes, see also updatehandle, Edit-Felder -> Slider
    update_sl = { { 'value','string','num2str(##)' } };
    update_sl = transpose(flipud(update_sl));
    % what should be updated when edit changes, see also updatehandle, Slider -> Edit-Felder    
    update_ed = { { 'string','value','str2num(##)' } };
    update_ed = transpose(flipud(update_ed));
    
    %Struktur fuer Slider
    Fu.uicontrol = struct( ...
	'frame',         { ll_frame }, ...
	'geometry',      { geom }, ...
	'xborder',       { [0.05, 0.05] }, ...
	'yborder',       { [0.1, 0.3] }, ...
	'xgap',          { [0.05] }, ...
	'ygap',          { [0.3] }, ...
	'xshift',        { [0.0] }, ...	
	'yshift',        { [0.0] }, ...	
	'xmodify',       { 'size' }, ...
	'parent',        { [1] }, ...
	'style',         { 'slider' }, ...
	'min',           { minval }, ...
	'max',           { maxval }, ...
	'value',         { svalue }, ...
	'visible',       { visible }, ...
	'interruptible', { 'off' }, ...
	'fontsize',      { 1 }, ...
	'string',        { str }, ...
	'callback',      { cab }, ...
	'tag',           { tag } ...
	);
    [HANDLE,slider_han] = gui_create(gui_define(Fu),HANDLE);
    % sets variable name in userdata + action and update information
    gui_set_userdata(slider_han,'variable',var); 
    gui_set_userdata(slider_han,'cbaction',cbaction);
    gui_set_userdata(slider_han,'cbchange',cbchange_sl);
    gui_set_userdata(slider_han,'updateproperty',update_sl); 
    gui_add_name(slider_han,'slider',' ','e','tag');    
    
    % add edit field shifted above (look for 'yshift')
    Fu.uicontrol = setfield(Fu.uicontrol,'xmodify',             'left');
    Fu.uicontrol = setfield(Fu.uicontrol,'xsize',               0.10);
    Fu.uicontrol = setfield(Fu.uicontrol,'yshift',              0.20);
    Fu.uicontrol = setfield(Fu.uicontrol,'style',               'edit');
    Fu.uicontrol = setfield(Fu.uicontrol,'horizontalalignment', 'left');    
    Fu.uicontrol = setfield(Fu.uicontrol,'string',              evalue);    
    Fu.uicontrol = setfield(Fu.uicontrol,'fontsize',            0.60);
    Fu.uicontrol = setfield(Fu.uicontrol,'callback',            'edit_check');
    %Fu.uicontrol = setfield(Fu.uicontrol,'tag',                 'slider_edit');
    Fu.uicontrol = setfield(Fu.uicontrol,'backgroundcolor',     ax_color);   
    [HANDLE,edit_han] = gui_create(gui_define(Fu),HANDLE);
    % sets variable name in userdata + action and update information
    gui_set_userdata(edit_han,'variable',var); 
    gui_set_userdata(edit_han,'cbaction',cbaction);
    gui_set_userdata(edit_han,'cbchange',cbchange_ed);
    gui_set_userdata(edit_han,'updateproperty',update_ed);
    gui_set_userdata(edit_han,'callback',[func_name,' ',cab]);
    gui_add_name(edit_han,'slider_edit',' ','e','tag');    

    % add text shifted above 
    Fu.uicontrol = setfield(Fu.uicontrol,'xmodify',             'right');
    Fu.uicontrol = setfield(Fu.uicontrol,'xsize',               0.15);
    Fu.uicontrol = setfield(Fu.uicontrol,'style',               'text');
    Fu.uicontrol = setfield(Fu.uicontrol,'horizontalalignment', 'right');    
    Fu.uicontrol = setfield(Fu.uicontrol,'string',              str);    
    Fu.uicontrol = setfield(Fu.uicontrol,'fontsize',            0.8);
    Fu.uicontrol = setfield(Fu.uicontrol,'callback',            '');
    %Fu.uicontrol = setfield(Fu.uicontrol,'tag',                 'slider_text');
    Fu.uicontrol = setfield(Fu.uicontrol,'backgroundcolor',     ax_color);   
    [HANDLE,text_han] = gui_create(gui_define(Fu),HANDLE);
    gui_add_name(text_han,'slider_text',' ','e','tag');    

    % updatehandle for slider (handles of edit)
    edit_cell = mat2cell(edit_han,1,ones(size(edit_han)));
    gui_set_userdata(slider_han,'updatehandle',edit_cell); 
    % updatehandle for edit (handles of slider)
    slider_cell = mat2cell(slider_han,1,ones(size(slider_han)));
    gui_set_userdata(edit_han,'updatehandle',slider_cell); 
    % adds function name to callback    
    gui_add_name([slider_han,edit_han],func_name,' ','b','callback');
    % adds common name to tag
    gui_add_name([slider_han,edit_han,text_han],'lower_left',' ','b','tag');
    % clears structur Fu    
    clear Fu
    
    
    % ----------------------------------------------------------------------  
    % lower right position, enthaelt Hilfe, Exit, Start und Stop Buttons
    % ----------------------------------------------------------------------  
    % basic axis for background
    Fbasis.axes = setfield(Fbasis.axes,'frame',lr_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);
    gui_add_name(newh,'lower_right',' ','b','tag');            

    % ----------------------------------------------------------------------  
    % define uicontrol pushbutton
    geom = [1,1 ; 1,1];
    str = reshape({ 'Start','Help', 'Stop', 'Exit'},size(geom));
    tag = lower(str);
    cab = lower(str);
    
    F.uicontrol = struct( ...
	'frame',    { lr_frame }, ...
	'geometry', { geom }, ...
	'parent',   { [1] }, ...
	'style',    { 'pushbutton' }, ...
	'visible',  { 'on' }, ...
	'fontsize', { 0.3 }, ...
	'string',   { str }, ...
	'callback', { cab }, ...
	'tag',      { tag } ...
	);
    [HANDLE,newh] = gui_create(gui_define(F),HANDLE); clear F;
    gui_add_name(newh,func_name,' ','b','callback');
    gui_add_name(newh,'lower_right',' ','b','tag');            
    % settings for stop handle
    stop_handle = gui_handle_query(newh,'tag','*stop');
    gui_set_userdata(stop_handle,'stop',0);
    set(stop_handle,'enable','off');
    % ----------------------------------------------------------------------  
    
    % ----------------------------------------------------------------------  
    % upper left position, enthaelt die Graphiken
    % ----------------------------------------------------------------------
    % basic axis for background
    Fbasis.axes = setfield(Fbasis.axes,'frame',ul_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);    
    gui_add_name(newh,'upper_left',' ','b','tag');            

    % ----------------------------------------------------------------------  
    % add axes for results
    geom = [3,3,3,1; 3,3,3,1; 3,3,3,1; 3,3,3,1];
    tag  = { ...
	    'imagemat','imagemat','imagemat','magnet_iteration';    ...
        'imagemat','imagemat','imagemat','magnet_temperatur';...
        'imagemat','imagemat','imagemat','energ_iteration';...
        'imagemat','imagemat','imagemat','energ_temperatur'};

    F.axes = struct( ...
	'frame',    { ul_frame }, ...
	'geometry', { geom }, ...
    'xgap',     { [0.05] }, ...
	'ygap',     { [0.05] }, ...
	'parent',   { [1] }, ...
    'fontsize', {0.1},...
	'visible',  { 'on' }, ...
	'box',      { 'off' }, ...
	'tag',      { tag } ...	
	);
    [HANDLE,axes_h] = gui_create(gui_define(F),HANDLE);clear F;
    gui_add_name(axes_h,'result','_','b','tag');
    gui_add_name(axes_h,'upper_left',' ','b','tag');
    image_axes = gui_handle_query(axes_h,'tag','*imagemat');
    gui_set_userdata(image_axes,'content','warm');
    gui_set_userdata(image_axes,'pick','warm');

    % add image
    axes_han   = HANDLE.axes;
    im_ax_han  = gui_handle_query(axes_han,'tag','*imagemat');
    figure_han = get(im_ax_han,'parent');    
    axes(im_ax_han);
    a_tag = get(im_ax_han,'tag');       % save tag
    user  = get(im_ax_han,'userdata');  % save userdata
    image_han = imagesc([],[],[]);
    set(im_ax_han,'tag',a_tag);         % restore tag
    set(image_han,'tag','imagemat');
    set(im_ax_han,'userdata',user);     % and userdata
    set(im_ax_han,'visible','off','clim',[0,1],'drawmode','fast');
    set(figure_han,'colormap',[ 1 0 0; 1 1 1 ]);

    axis image;
    HANDLE = gui_create(image_han,HANDLE);    
    
    % add lines to axes (magnet_iteration)
    lax_h     = gui_handle_query(axes_h,'tag','*magnet_iteration');
    tag       = { 'magnetisierung_iteration'};
    col       = { 'b'};
    linestyle   = { '-' };
    linewidth = { 2 }; 
    markerstyle = { 'none' };
    markersize  = {  2 };
    GUI.line  = struct( 'parent', { lax_h }, ...
	'xdata',     { [] }, ...
	'ydata',     { [] }, ...
	'linewidth', linewidth, ...
    'linestyle',    linestyle, ...
	'marker',       markerstyle, ...
	'markersize',   markersize, ...
	'erasemode', 'xor', ...
	'color',     col, ...
	'tag',       tag ...
	);
    [HANDLE,line_x_h] = gui_create(GUI,HANDLE); clear GUI
    
    % add lines to axes (energ_iteration)
    lax_h = gui_handle_query(axes_h,'tag','*energ_iteration');
    tag   = { 'energie_iteration' };
    col       = { 'r'};
    linestyle   = { '-' };
    linewidth   = {  2  };
    markerstyle = { 'none' };
    markersize  = {  2 };
    GUI.line = struct( 'parent', { lax_h }, ...
	'xdata',     { [] }, ...
	'ydata',     { [] }, ...
	'linewidth', linewidth, ...
    'linestyle',    linestyle, ...
	'marker',       markerstyle, ...
	'markersize',   markersize, ...
	'erasemode', 'xor', ...
	'color',     col, ...
	'tag',       tag ...
	);
    [HANDLE,line_y_h] = gui_create(GUI,HANDLE); clear GUI
    
    % add lines to axes (magnet_temperatur)
    lax_h = gui_handle_query(axes_h,'tag','*magnet_temperatur');
    tag         = { 'magnetisierung_temperatur','magnetisierung_exakt' };
    col         = { 'b','g' };
    linestyle   = { 'none','-' };
    linewidth   = {  3,2  };
    markerstyle = { 'o','none' };
    markersize  = {  2,2 };
    GUI.line = struct( 'parent', { lax_h }, ...
	'xdata',        { [] }, ...
	'ydata',        { [] }, ...
	'linewidth',    linewidth, ...
	'linestyle',    linestyle, ...
	'marker',       markerstyle, ...
	'markersize',   markersize, ...
	'erasemode',    'xor', ...
	'color',        col, ...
	'tag',          tag ...
	);
    [HANDLE,line_i_h] = gui_create(GUI,HANDLE); clear GUI
     
    % add lines to axes (energ_temperatur)
    lax_h = gui_handle_query(axes_h,'tag','*energ_temperatur');
    tag         = { 'energie_temperatur','energie_exakt'  };
    col         = { 'r','g'};
    linestyle   = { 'none','-'};
    linewidth   = { 3,2  };
    markerstyle = {'o','none' };
    markersize  = { 2,2  };
    GUI.line = struct( 'parent', { lax_h }, ...
	'xdata',        { [] }, ...
	'ydata',        { [] }, ...
	'linewidth',    linewidth, ...
	'linestyle',    linestyle, ...
	'marker',       markerstyle, ...
	'markersize',   markersize, ...
	'erasemode',    'xor', ...
	'color',        col, ...
	'tag',          tag ...
	);
    [HANDLE,line_i_h] = gui_create(GUI,HANDLE); clear GUI
    % ----------------------------------------------------------------------  
    
    % ----------------------------------------------------------------------  
    % upper right position
    % ----------------------------------------------------------------------  
    % basic axis for background
    Fbasis.axes = setfield(Fbasis.axes,'frame',ur_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);
    gui_add_name(newh,'upper_right',' ','b','tag');            

    % pushbutton
    geom = ones(6,1);
    str = reshape( ...
	{ 'Warm','Cold','Accept','Resume','Discard-all', ...
	'Metropolis'}, ...
	size(geom));
    sty=reshape( ...
	{ 'pushbutton','pushbutton','pushbutton','pushbutton','pushbutton', ...
    'togglebutton'}, ...
	size(geom));
    str1 = reshape( ...
	{ 'Warm','Cold','Accept','Resume','Discard-all', ...
	'Clustering'}, ...
	size(geom));
    tag = lower(str1);
    
    cab1     = 'setup_imagemat';
    cab_accept     = 'accept';
    cab_resume = 'resume';
    cab_discard_all  = 'discard_all';
    cab_clustering  = 'clustering';

    
    cab  = reshape( ... 
	{ cab1, cab1, cab_accept, cab_resume, cab_discard_all, ...
	cab_clustering}, ...
 	size(geom));
    
    Fpush.uicontrol = struct( ...
	'frame',    { ur_frame }, ...
	'geometry', { geom }, ...
	'yborder',  { [0.25,0.05] }, ...
	'ygap',     { [0.02] }, ...
	'parent',   { [1] }, ...
	'style',    { sty }, ...
	'visible',  { 'on'}, ...
	'fontsize', { 0.3 }, ...
	'string',   { str }, ...
	'callback', { cab }, ...
	'tag',      { tag } ...
	);
    [HANDLE,newh] = gui_create(gui_define(Fpush),HANDLE); clear Fpush;
    gui_add_name(newh,func_name,' ','b','callback');
    gui_add_name(newh,'upper_right',' ','b','tag');
    reset_han = gui_handle_query(newh,'tag','*reset');
    set(reset_han,'enable','off')
   
    % ----------------------------------------------------------------------  

    % ----------------------------------------------------------------------  
    % keep this here to setup first image 
    % ----------------------------------------------------------------------  
    eval( [handle_name,'=HANDLE;'] );
    eval( [func_name,' ''setup_imagemat'''] );
    
    % ----------------------------------------------------------------------  
    % postprocessing for initgraph (keep this lines)
    % ----------------------------------------------------------------------  
    eval( [func_name,' ''initgraph_post'''] );
    % ----------------------------------------------------------------------  
    
    % ----------------------------------------------------------------------  
    % set main figure to visible on (keep this lines)   
    % ----------------------------------------------------------------------  
    fig_han = eval([handle_name,'.figure']);   % get content of figure
    fig_han = gui_handle_query(fig_han,'tag',['*main']);
    set(fig_han,'visible','on');          % set figures to visible
    % ======================================================================
  case 'initgraph_pre'               
    % preprocessing for initgraph (keep this lines)    
    % ======================================================================
    gui_default('set') % default values
    % close figures belonging to this program
    fig_han = get(0,'children');   
    fig_han = gui_handle_query(fig_han, ...
	'type','figure','tag',[func_name,'*']);
    delete(fig_han);
    % ======================================================================
  
  case 'initgraph_post'               
    % postprocessing for initgraph (keep this lines)   
    % ======================================================================
    gui_default('remove'); % reset defaults to MATLAB defaults
    % ======================================================================
  
  case 'exit'
  % finishing the program (keep this lines)
  % ========================================================================
    fig_han = eval([handle_name,'.figure']); % get all figures
    fig_han = fig_han(ishandle(fig_han));    % check if exist as handle
    delete(fig_han);                         % closes all figures
    clear('global',handle_name)              % deletes the global variable
    % ======================================================================
  
  case 'close_request_main'  
    % close request main (keep this lines)
    % ======================================================================
    func = gui_get_userdata(gcf,'function'); % function name
    exit = gui_get_userdata(gcf,'end');      % action for exit
    msg = { ...
	['Closing this figure exits program ',func,'.']; ...
	'Do you really want to exit?' ...
	};
    tit = 'Close Request Function';
    selection = questdlg(msg,tit,'Yes','No','Yes');
    switch selection,
      case 'Yes',
	eval( [func,' ',exit] )
      case 'No'
	return
    end
    % ======================================================================
    
  case 'close_request_help'  
    % close request for help (keep this lines)
    % ======================================================================
    func = gui_get_userdata(gcf,'function');
    exit = gui_get_userdata(gcf,'end');
    msg = { ...
	'Closing this figure is not possible.'; ...
	'Do you want to make it invisible?' ...
	};
    tit = 'Close Request Function';
    selection = questdlg(msg,tit,'Yes','No','Yes');
    switch selection,
      case 'Yes',
	eval( [func,' ',exit] )
      case 'No'
	return
    end    
    % ======================================================================
        
  case 'help'
    % calling help
    % ======================================================================
    frame = [0.1, 0.1, 0.8, 0.5];
    [HANDLE,new] = gui_help(HANDLE,frame,func_name);
    if new, eval([handle_name,'=HANDLE;']); end % export to global
    % ======================================================================
  
  case 'edit_check'
    % checks string from edit field
    % ======================================================================
    str    = get(gcbo,'string');
    oldval = get(gcbo,'value');
    minval = get(gcbo,'min');
    maxval = get(gcbo,'max');
    val = str2num(str);
    % handle all error cases
    if ~isempty(val)
      if val<minval
	msg = { 'Value to small!','Value set to minimum.' };
	e_han = msgbox(msg,'Edit Box','error','modal');
	set(gcbo,'string',num2str(minval));
      elseif val>maxval
	msg = { 'Value to large!','Value set to maximum.' };
	e_han = msgbox(msg,'Edit Box','error','modal');
	%e_han = errordlg(msg,'Edit Box','on');
	set(gcbo,'string',num2str(maxval));
      end
    else
      msg = { 'Wrong value!','Value set to old value.' };
      e_han = msgbox(msg,'Edit Box','error','modal');
      %e_han = errordlg(msg,'Edit Box','on');
      if isfield(get(gcbo,'userdata'),'updatehandle')
	updatehandle   = gui_get_userdata(gcbo,'updatehandle');      
	updateproperty = gui_get_userdata(gcbo,'updateproperty');
	oldval = get(updatehandle,updateproperty{2});
      end
      set(gcbo,'string',num2str(oldval));
    end
    
    % do the real callback
    if isfield(get(gcbo,'userdata'),'callback')
      eval( gui_get_userdata(gcbo,'callback') );
    end
    % ======================================================================
    
    
  case 'set_value'
    % get values from caller 
    % ======================================================================
    e_str  = gui_get_userdata(gcbo,'variable');
    cbaction = gui_get_userdata(gcbo,'cbaction');
    figure_han = get(gcbo,'parent');
    func_name = gui_get_userdata(figure_han,'function');
    
    if isfield(get(gcbo,'userdata'),'cbchange')
      cbchange = gui_get_userdata(gcbo,'cbchange');
      if ~isempty(cbchange)
	prop = cbchange{1};
	chfunc = cbchange{2};
	e_str = [ 'get(gcbo,''',prop,''')' ];
	e_str = strrep(chfunc,'##',e_str);
	e_str = [ 'set(gcbo,''',prop,''',',e_str,');' ];
	eval( e_str );
      end
    end
    
    % do the update of another handle
    if isfield(get(gcbo,'userdata'),'updatehandle')
      updatehandle   = gui_get_userdata(gcbo,'updatehandle');      
      updateproperty = gui_get_userdata(gcbo,'updateproperty');      
      if ~isempty(updatehandle)
	from_prop = updateproperty{1};
	to_prop   = updateproperty{2};
	chfunc    = updateproperty{3};
	e_str = [ 'get(gcbo,''',from_prop,''')' ];
	e_str = strrep(chfunc,'##',e_str);
	e_str = [ 'set(updatehandle,''',to_prop,''',',e_str,');' ];
	eval( e_str );
      end
    end
    
    if ~isempty(cbaction)
      eval( [func_name,' ',cbaction] );
    end
    % ======================================================================

  case {'start','resume'}
    % start simulation
    % ======================================================================
    % get variables from slider values
    uicontrol_han = HANDLE.uicontrol;
    %sucht alle Handles, die zu den Sliders gehren
    data_handle = gui_handle_query(uicontrol_han,'tag','*slider');
    % set all variables
    variables = gui_get_userdata(data_handle,'variable');
    values    = get(data_handle,'value');
    lv = length(values);
    if lv==1
      eval([variables,'=values;']);
    else
      for k = 1:lv
	value = values{k};
	var   = variables{k};
	eval([var,'=value;']);
      end
    end
    
    % get matrix from image
    image_han = HANDLE.image; 
    image_han = gui_handle_query(image_han,'tag','imagemat');
    spin_matrix  = get(image_han,'cdata');
    %ersetze null durch -1 in der Spinmatrix
    spin_matrix(find(spin_matrix==0))=-1;
    % find handle of stop button and of buttons to be disabled
    stop_han = gui_handle_query(uicontrol_han,'tag','*stop');    
    %alle nicht-Stop Handles werden deak
    disable_han = uicontrol_han(uicontrol_han~=stop_han);
    enable_status = {get(disable_han,'enable')};
    gui_set_userdata(gcbo,'enable_status',enable_status);        
    % disable buttons and sliders, enable stop button
    set(disable_han,'enable','off');
    set(stop_han,'enable','on');
    
    % make calculations
    % create update handle as needed in ising_it
    up_han     = image_han;
    axes_han   = HANDLE.axes;
    ax_h_mag_it = gui_handle_query(axes_han,'tag','*magnet_iteration');
    up_han     = [up_han,gui_get_userdata(ax_h_mag_it,'anim_handle')];
    ax_h_energ_it = gui_handle_query(axes_han,'tag','*energ_iteration');
    up_han     = [up_han,gui_get_userdata(ax_h_energ_it,'anim_handle')];
    ax_h_mag_temp  = gui_handle_query(axes_han,'tag','*magnet_temperatur');
    up_han     = [up_han,gui_get_userdata(ax_h_mag_temp,'anim_handle')];
    ax_h_energ_temp  = gui_handle_query(axes_han,'tag','*energ_temperatur');
    up_han     = [up_han,gui_get_userdata(ax_h_energ_temp,'anim_handle')];
    
    % get some specific line handles
    lin_han = HANDLE.line;
    lin_han = gui_handle_query(lin_han,'parent',ax_h_mag_it);
    ite_h   = gui_handle_query(lin_han,'tag','*magnetisierung_iteration');
   
    uicontrol_han=HANDLE.uicontrol;
    ui_h_clustering=gui_handle_query(uicontrol_han,'tag','*clustering*');
    cluster_faktor=get(ui_h_clustering,'value');
    
    % adjust iteration axes
    switch action
    case 'start'
        resume_faktor=0;
        set(ax_h_mag_it,'xlim',[1,iterations])    
        set(ax_h_energ_it,'xlim',[1,iterations])

        % do iteration, Aufruf des eigentlichen Ising-Programms, welches die Berechnungen durchfuehrt
        [spin_matrix,k]=ising_it(spin_matrix, iterations, temperature, time_delay, resume_faktor, cluster_faktor, up_han, stop_han); 

    case 'resume'  %wenn Magnetisierung nicht auskonvergiert ist, werden weitere Berechnungen durchgefuehrt
        resume_faktor=1;
        set(ax_h_mag_it,'xlim',[1,iterations+length(get(up_han(2),'xdata'))])    
        set(ax_h_energ_it,'xlim',[1,iterations+length(get(up_han(3),'xdata'))])
        [spin_matrix,k]=ising_it(spin_matrix, iterations, temperature, time_delay, resume_faktor, cluster_faktor, up_han, stop_han); 
    end
    
    set(ax_h_mag_it,'xlim',[1,k])    
    set(ax_h_energ_it,'xlim',[1,k])
    
    % enable buttons, disable stop button     
    gui_set_userdata(stop_han,'stop',0);
    set(stop_han,'enable','off');
    enable_status = gui_get_userdata(gcbo,'enable_status');
    for k = 1:length(disable_han)
      set(disable_han(k),'enable',enable_status{k});
    end
    % ======================================================================
  case 'stop'
    % stop
    % ======================================================================
    stop_han = gui_handle_query(HANDLE.uicontrol,'tag','*stop');    
    gui_set_userdata(stop_han,'stop',1);
    % ======================================================================
    
  case 'setup_imagemat'    
    % setup of matrix
    % ======================================================================
    % check all sliders and set variables
    % find axes, figure and image handle
    image_han = HANDLE.image;
    image_han = gui_handle_query(image_han,'tag','*imagemat');
    axes_han   = HANDLE.axes;
    im_ax_han  = get(image_han,'parent');
    figure_han = get(im_ax_han,'parent');
        
    % determine content of axes with image
    content = gui_get_userdata(im_ax_han,'content');
    gcbo_style = get(gcbo,'style');
    if ~isempty(gcbo_style)
      if strcmp(gcbo_style,'pushbutton')
	content = get(gcbo,'string');
	gui_set_userdata(im_ax_han,'content',content); 
      end
    end  
    
    % set all variables
    uicontrol_han = HANDLE.uicontrol;
    data_handle = gui_handle_query(uicontrol_han,'tag','*slider');
    variables = gui_get_userdata(data_handle,'variable');
    values    = get(data_handle,'value');
    lv = length(values);
    if lv==1
      eval([variables,'=values;']);
    else
      for k = 1:lv
	value = values{k};
	var   = variables{k};
	eval([var,'=value;']);
      end
    end

    switch lower(content(1:4))
      case 'warm'  
	imagemat = randarr(initial_magnet,y_dimension,x_dimension);
	%gui_set_userdata(im_ax_han,'pick','warm');
      case 'cold'
	imagemat = zeros(y_dimension,x_dimension);
	%gui_set_userdata(im_ax_han,'pick','cold');
       end
      %Matrix mit 1 (Spin_up) und -1 (Spin_down) statt 0 und 1 (besser fuer die Berechnung der Magnetisierung)
      spin_matrix=imagemat;
      spin_matrix(find(spin_matrix==0))=-1;

      % get image data 
    image_size = size(imagemat);
    x_vec      = [1:image_size(2)];
    y_vec      = [1:image_size(1)];
    tot_sum    = sum(imagemat(:));

    % update image
    axes(im_ax_han);
    set(image_han,'erasemode','normal');
    set(image_han, ...
	'parent',im_ax_han,'xdata',x_vec,'ydata',y_vec,'cdata',imagemat);
    axis image; 
    set(image_han,'erasemode','none');
    % sets the animation data
    gui_set_userdata(im_ax_han,'anim_handle',image_han);
    
    % draw lines (magnet_iteration)
    li_ax_han   = gui_handle_query(axes_han,'tag','*magnet_iteration');
    axes(li_ax_han);
    %Berechnung der akzuellen Magnetisierung
    magnetisierung_iteration=sum(sum(spin_matrix))/prod(image_size);   
    set(gca,'xlim',[1,iterations],'ylim',[-1.05,1.05]);
    xlabel('Iterations');
    ylabel('Magnetisation');
    lin_han = HANDLE.line;
    lin_han = gui_handle_query(lin_han,'parent',gca);
    mag_it_h  = gui_handle_query(lin_han,'tag','*magnetisierung_iteration');
    %setzen der x- und y Daten
    set(mag_it_h, 'xdata',2, 'ydata',magnetisierung_iteration);
    gui_set_userdata(gca,'anim_handle',[mag_it_h]);
    grid on

    % draw lines (energ_iteration)
    li_ax_han   = gui_handle_query(axes_han,'tag','*energ_iteration');
    axes(li_ax_han);
    
    [spin_up_down_period,pos]=matrix_periodisch(spin_matrix);
    for untergitter=1:2
        energie_iteration(untergitter)=mean(mean(energie(spin_up_down_period,pos(untergitter,:))));      
    end
    energie_iteration=mean(energie_iteration);
    set(gca,'xlim',[1,iterations],'ylim',[-4.1,4.1]);
    xlabel('Iterations');
    ylabel('Energy');
    lin_han = HANDLE.line;
    lin_han = gui_handle_query(lin_han,'parent',gca);
    energ_it_h  = gui_handle_query(lin_han,'tag','*energie_iteration');
    set(energ_it_h, 'xdata',2, 'ydata',energie_iteration);
    gui_set_userdata(gca,'anim_handle',[energ_it_h]);
    grid on

    % draw lines (magnet_temperatur)
    li_ax_han   = gui_handle_query(axes_han,'tag','*magnet_temperatur');
    axes(li_ax_han);
    magnetisierung_temperatur=sum(sum(spin_matrix))/prod(image_size); 
    %Berechnung der exakten Energie
    temperatur_intervall=0.001:0.001:5;
    temperatur_kritisch=2.269;
    magnetisierung_exakt(temperatur_intervall<=5)=0;
    magnetisierung_exakt(temperatur_intervall<=temperatur_kritisch)...
        =(1-1./(sinh(2./(temperatur_intervall(temperatur_intervall<=temperatur_kritisch)))).^4).^(1/8);

    set(gca,'xlim',[0,5],'ylim',[-0.02,1.02]);
    xlabel('Temperature');
    ylabel('Magnetisation');
    lin_han = HANDLE.line;
    lin_han = gui_handle_query(lin_han,'parent',gca);
    mag_temp_h  = gui_handle_query(lin_han,'tag','*magnetisierung_temperatur');
    mag_temp_exakt_h  = gui_handle_query(lin_han,'tag','*magnetisierung_exakt');

    xdata_vorher=get(mag_temp_h,'xdata');
    ydata_vorher=get(mag_temp_h,'ydata');        
    if length(xdata_vorher)>1
        set(mag_temp_h, 'xdata',[xdata_vorher(1:end-1),temperature], 'ydata', [ydata_vorher(1:end-1),abs(magnetisierung_temperatur)]);
    else
        set(mag_temp_h, 'xdata',temperature, 'ydata', abs(magnetisierung_temperatur));
    end
              
    set(mag_temp_exakt_h, 'xdata',temperatur_intervall, 'ydata', magnetisierung_exakt);              


    gui_set_userdata(gca,'anim_handle',[mag_temp_h,mag_temp_exakt_h ]);
    grid on

    % draw lines (energ_temperatur)
    %Berechnung der exakten temperaturabhaengigen Energie
    J=2;
    i=0;
    maxT=5;
    temperatur_intervall=0.1:0.1:5;
    for T=0.1:0.1:maxT
        i=i+1;    
        L=J/(2*T);
        K1=quad(@energie_exakt,0,pi/2,[],[],T,J);
        kappa1_2strich=2*(tanh(2*L)).^2-1;
        energie_exakt(i)=-2*(J/2)*coth((2*L)).*(1+(2/pi)*kappa1_2strich*K1);
    end

    li_ax_han   = gui_handle_query(axes_han,'tag','*energ_temperatur');
    axes(li_ax_han);
    %Berechnung der mittleren Energie pro Untergitter
        for untergitter=1:2
        energie_iteration(untergitter)=mean(mean(energie(spin_up_down_period,pos(untergitter,:))));
    end
    energie_temperatur=mean(energie_iteration);
    set(gca,'xlim',[0,5],'ylim',[-4.1,4.1]);
    xlabel('Temperature');
    ylabel('Energy');
    lin_han = HANDLE.line;
    lin_han = gui_handle_query(lin_han,'parent',gca);
    energ_temp_h  = gui_handle_query(lin_han,'tag','*energie_temperatur');
    energ_temp_exakt_h  = gui_handle_query(lin_han,'tag','*nergie_exakt');
    %ermittelt aus den Plots Temperatur und Energie, fuegt die neuen Werte hinzu und setzt sie
    xdata_vorher=get(energ_temp_h,'xdata');
    ydata_vorher=get(energ_temp_h,'ydata'); 
        
    if length(xdata_vorher)>1
        set(energ_temp_h, 'xdata',[xdata_vorher(1:end-1),temperature], 'ydata', [ydata_vorher(1:end-1),energie_temperatur]);
    else
        set(energ_temp_h, 'xdata',temperature, 'ydata', energie_temperatur);
    end
    set(energ_temp_exakt_h, 'xdata',temperatur_intervall, 'ydata',energie_exakt);

    gui_set_userdata(gca,'anim_handle',[energ_temp_h,energ_temp_exakt_h]);
    grid on

  % ======================================================================
case {'accept','discard_all'}
    %accept nimmt Energie und Magnetisierung der aktuellen Temperatur auf und speichert sie
    %discard all loescht alle Eintraege in den temperaturabhaengigen Plots
    image_han = HANDLE.image; 
    axes_han   = HANDLE.axes;

    up_han     = [];
    ax_h_mag_temp  = gui_handle_query(axes_han,'tag','*magnet_temperatur');
    up_han     = [up_han,gui_get_userdata(ax_h_mag_temp,'anim_handle')];
    ax_h_energ_temp  = gui_handle_query(axes_han,'tag','*energ_temperatur');
    up_han     = [up_han,gui_get_userdata(ax_h_energ_temp,'anim_handle')];
 

    mag_temp_h          = up_han(1);    %mag_temp_h ist der 1. Handle in up_han (Animationen)
    energ_temp_h        = up_han(3);
    
    mag_xdata=get(mag_temp_h,'xdata');
    mag_ydata=get(mag_temp_h,'ydata');
    energ_xdata=get(energ_temp_h,'xdata');
    energ_ydata=get(energ_temp_h,'ydata');
    switch action
        
    case 'accept'
         %fragt ab, ob es fuer eine Temperatur 2 Eintraege gibt        
        anzahl=find(mag_xdata==mag_xdata(end));
        if length(anzahl)>1
            %wenn ja, dann wird der erste Wert fuer die Temperatur auf jenen des neuen gesetzt
            mag_ydata(min(anzahl))=mag_ydata(end);
            mag_xdata(end)=inf;
            mag_ydata(end)=inf;
            energ_ydata(min(anzahl))=energ_ydata(end);
            energ_xdata(end)=inf;
            energ_ydata(end)=inf;
        else
            %formal wird ein neuer Eintrag hinzugefuegt, der spaeter wieder geloescht wird
            mag_xdata(end+1)=inf;
            mag_ydata(end+1)=inf;
            energ_xdata(end+1)=inf;
            energ_ydata(end+1)=inf;
        end
        %aufsteigend Sortieren nach der Temperatur
        [mag_xdata,h1]=sort(mag_xdata);
        mag_ydata=mag_ydata(h1);
        [energ_xdata,h1]=sort(energ_xdata);
        energ_ydata=energ_ydata(h1);        
                   
    case 'discard_all'
        %alle Daten werden geloescht
        mag_xdata=[];            
        mag_ydata=[];
        energ_xdata=[];
        energ_ydata=[];        
    end

    set(mag_temp_h,'xdata',mag_xdata,'ydata',mag_ydata);
    set(energ_temp_h,'xdata',energ_xdata,'ydata',energ_ydata);
    
case 'clustering'
    %Text des Togglebuttons wird gesetzt, ebenso die Farbe der Spinmatrix
    uicontrol_han=HANDLE.uicontrol;
    ui_h_clustering=gui_handle_query(uicontrol_han,'tag','*clustering*');
    cluster_ja_nein=get(ui_h_clustering,'value');
    switch cluster_ja_nein
    case 0
    set(ui_h_clustering,'string','Metropolis') 
    colormap([ 1 0 0; 1 1 1]);
    case 1
    set(ui_h_clustering,'string','Clustering') 
    colormap([ 1 1 0; 0 0 0]);
    end
 % ======================================================================
    

  otherwise
    % ======================================================================
    % warnings if callback routine is not defined (keep this at the end)    
    % ======================================================================
    disp(['Callback not defined: ',action]);
    % ======================================================================

end