function [rmat] = randarr(varargin)
%
% Creates an integer array of randomly distributed ones.
% 
% rmat = randarr(bound,n) is an n-by-n matrix of zeros and ones
% rmat = randarr(bound,m,n) is an m-by-n matrix of zeros and ones
% rmat = randarr(bound,[m,n]) is an m-by-n matrix of zeros and ones
% rmat = randarr(bound,m,n,p,..) is an m-by-n-by-p matrix of zeros and ones
%
% bound:   0<= bound <= 1 - relative number of zeros in rmat
%
% Author:  Winfried Kernbichler
% Version: 1.1 
% Date:    October 20, 2001
% Status:  Works
% Missing: Check of inputs for consistency
%
bound = varargin{1};
if bound < 0 | bound > 1, error('Wrong boundary [0:1]'); end
dims = [varargin{2:end}];
rmat                = rand(dims);
rmat(rmat <= bound) = 0;
rmat(rmat >  bound) = 1;